require("GUIS")

function love.load()
	shader = love.graphics.newShader("mandelbrot.frag")
	
	Render = {
		ResolutionX = GUIS.newIntegerSlider  ("ResolutionX",130,75,270,15,0,4096,4096),
		ResolutionY = GUIS.newIntegerSlider  ("ResolutionY",130,90,270,15,0,4096,4096)
	}
	
	Width,Height = love.graphics.getDimensions()
	
	Fractal = {
		Iterations = GUIS.newIntegerSlider("Iterations.Value",130,30,270,15,0,1024,256)
	}
	
	Window = {
		ZoomSpeed = 1.05,
		Zoom = 1,
		Pan = {x = 0,y = 0},
		
		PanTowardMouse = function(self, mx, my, s)
			newWindowZoom = self.Zoom*s
		
			-- Transform mouse coords to zoomed graphic axes
			mx = (mx * 2 - Width) * 1.5 / Height
			my = (my * 2 - Height) * 1.5 / Height

			-- Calculate new centre so that the mouse remains in the same place
			self.Pan.x = self.Pan.x + mx * self.Zoom - mx * newWindowZoom
			self.Pan.y = self.Pan.y + my * self.Zoom - my * newWindowZoom
			
			self.Zoom = newWindowZoom 
		end
	}

	ShowInfo = true
	GUISInteract = false
end

function UpdateShader()
	--pcall(function()
		shader:send('iResolution',{Width,Height})
		--shader:send('iGlobalTime',love.timer.getTime())
		shader:send('Iterations',Fractal.Iterations.Value)
		shader:send('Zoom',Window.Zoom)
		shader:send('Pan',{Window.Pan.x,Window.Pan.y})
	--end)
end


function love.resize(w, h)
  print(("Window resized to width: %d and height: %d."):format(w, h))
  Width = w
  Height = h
end

function love.keypressed(key)
	if key == "b" then
		RenderCanvas = love.graphics.newCanvas(Render.ResolutionX.Value,Render.ResolutionY.Value,"rgba32f")
		
		shader:send('iResolution',{Render.ResolutionX.Value,Render.ResolutionY.Value})
		
		love.graphics.setCanvas(RenderCanvas)
			love.graphics.setShader(shader)
				love.graphics.rectangle("fill",0,0,Render.ResolutionX.Value,Render.ResolutionY.Value)
			love.graphics.setShader()
		love.graphics.setCanvas()
		
		RenderCanvas:newImageData():encode('png', os.time() .. '.png');
		
		print("screen shot taken")
	elseif key=="i" then
		ShowInfo = not ShowInfo
	end
end



function love.mousepressed(x,y,button)
	if(ShowInfo) then
		if(GUIS:mousepressed(x,y,button)) then
			GUISInteract = true
		end
	end
end

function love.mousemoved(x,y,dx,dy)
	GUIS:mousemoved(x,y,dx,dy)
end

function love.mousereleased(x,y,button)
	GUIS:mousereleased(x,y,button)
	GUISInteract = false
end

function love.update(dt)
	UpdateShader()
	
	if (not GUISInteract) then
		local mx, my = love.mouse.getPosition()
		
		if love.mouse.isDown(1) then
			Window:PanTowardMouse(mx,my,1/Window.ZoomSpeed)
		end
		if love.mouse.isDown(2) then
			Window:PanTowardMouse(mx,my,Window.ZoomSpeed)
		end
	end

	GUIS:update()
end

function love.draw()
	love.graphics.setShader(shader)
	love.graphics.rectangle("fill",0,0,Width,Height)	
	love.graphics.setShader()
	if (ShowInfo) then
		love.graphics.setColor(0,0,0)
		love.graphics.rectangle("fill",0,0,400,105)
		love.graphics.setColor(255,255,255)
		love.graphics.print("press 'b' to make a screenshot",0,0)
		love.graphics.print("press 'i' to remove this info",0,15)
		love.graphics.print("Iterations :",0,30)
		love.graphics.print("Pan :"..Window.Pan.x..","..Window.Pan.y,0,45)
		love.graphics.print("Zoom :"..Window.Zoom,0,60)
		love.graphics.print("Render Resolution :",0,75)
		
		GUIS:draw()
	end
end